@@ -4,7 +4,7 @@
  * Copyright (C) 2012 Paul Wouters <pwouters@redhat.com>
  * Copyright (C) 2004 Michael Richardson <mcr@xelerance.com>
  * Copyright (C) 1997 Angelos D. Keromytis.
- * Copyright (C) 1998-2002  D. Hugh Redelmeier.
+ * Copyright (C) 1998-2002,2014  D. Hugh Redelmeier.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -61,11 +61,13 @@
 	"5E2327CF EF98C582 664B4C0F 6CC41659"
 #endif
 
-#define MODP768_MODULUS \
+#if defined(USE_VERYWEAK_DH1)
+# define MODP768_MODULUS \
     "FFFFFFFF FFFFFFFF C90FDAA2 2168C234 C4C6628B 80DC1CD1 " \
     "29024E08 8A67CC74 020BBEA6 3B139B22 514A0879 8E3404DD " \
     "EF9519B3 CD3A431B 302B0A6D F25F1437 4FE1356D 6D51C245 " \
     "E485B576 625E7EC6 F44C42E9 A63A3620 FFFFFFFF FFFFFFFF"
+#endif
 
 #define MODP1024_MODULUS \
     "FFFFFFFF FFFFFFFF C90FDAA2 2168C234 C4C6628B 80DC1CD1 " \
@@ -386,6 +388,8 @@ enum next_payload_types {
 	ISAKMP_NEXT_NATOA_DRAFTS = 131   /* NAT-Traversal: NAT-OA (drafts) */
 };
 
+#define ISAKMP_v2PAYLOAD_TYPE_BASE     ISAKMP_NEXT_v2SA        /* lowest value of a v2 payload type */
+
 /* These values are to be used within the Type field of an Attribute (14) 
  *    ISAKMP payload.  */
 #define ISAKMP_CFG_REQUEST         1
