@@ -40,21 +40,6 @@ static const char *const version_name_1[] = {
 };
 static const char *const version_name_2[] = {
 	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
-	"IKEv2 version 2.0 (rfc4306/rfc5996)",
 };
 
 enum_names version_names_1 =
@@ -62,10 +47,9 @@ enum_names version_names_1 =
 	ISAKMP_MAJOR_VERSION<<ISA_MAJ_SHIFT | ISAKMP_MINOR_VERSION,
 	version_name_1, NULL };
 
-/* To ignore or (in other words, accept) all IKEv2 minor versions*/
 enum_names version_names =
-    { IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | 0x0,
-	IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | 0xf,
+    { IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | IKEv2_MINOR_VERSION,
+	IKEv2_MAJOR_VERSION<<ISA_MAJ_SHIFT | IKEv2_MINOR_VERSION,
 	version_name_2, &version_names_1 };
 
 /* Domain of Interpretation */
@@ -162,7 +146,8 @@ const char *const payload_name[] = {
 	NULL
     };
 
-const char *const payload_names_ikev2[] = {
+/* dual-use: for enum_name and for bitnamesof */
+const char *const payload_name_ikev2_main[] = {
     "ISAKMP_NEXT_v2SA",            /* 33 */
     "ISAKMP_NEXT_v2KE",
     "ISAKMP_NEXT_v2IDi",
@@ -179,23 +164,23 @@ const char *const payload_names_ikev2[] = {
     "ISAKMP_NEXT_v2E",
     "ISAKMP_NEXT_v2CP",
     "ISAKMP_NEXT_v2EAP",
-    NULL
+    NULL    /* termination for bitnamesof() */
 };
 
-const char *const payload_name_nat_d[] = {
-    "ISAKMP_NEXT_NAT-D",
-    "ISAKMP_NEXT_NAT-OA",
-    NULL
+static enum_names payload_names_ikev2_main =
+{ ISAKMP_NEXT_v2SA, ISAKMP_NEXT_v2EAP, payload_name_ikev2_main,
+  NULL };
+
+const char *const payload_name_ikev2[] = {
+    "ISAKMP_NEXT_v2NONE",            /* 33 */
 };
 
-static enum_names payload_names_nat_d =
-{ ISAKMP_NEXT_NATD_DRAFTS, ISAKMP_NEXT_NATOA_DRAFTS, payload_name_nat_d, NULL };
-static enum_names payload_names_ikev2_d =
-{ ISAKMP_NEXT_v2SA, ISAKMP_NEXT_v2EAP, payload_names_ikev2, &payload_names_nat_d };
+enum_names payload_names_ikev2 =
+{ ISAKMP_NEXT_NONE, ISAKMP_NEXT_NONE, payload_name_ikev2,
+  &payload_names_ikev2_main };
 
 enum_names payload_names =
-{ ISAKMP_NEXT_NONE, ISAKMP_NEXT_NATOA_RFC, payload_name, &payload_names_ikev2_d };
-
+{ ISAKMP_NEXT_NONE, ISAKMP_NEXT_NATOA_RFC, payload_name, &payload_names_ikev2_main };
 
 /* Exchange types (note: two discontinuous ranges) */
 
@@ -283,10 +268,11 @@ enum_names isakmp_transformid_names =
 
 static const char *const ah_transform_name_private_use[] = {
 	"AH_NULL", /* verify with kame source? 251 */
+	"AH_SHA2_256_TRUNC", /* our own to signal bad truncation to kernel */
     };
 
 enum_names ah_transformid_names_private_use =
-    { AH_NULL, AH_NULL, ah_transform_name_private_use, NULL };
+    { AH_NULL, AH_SHA2_256_TRUNC, ah_transform_name_private_use, NULL };
 
 
 static const char *const ah_transform_name[] = {
