@@ -2,6 +2,7 @@
  * Copyright (C) 1999-2001  D. Hugh Redelmeier
  * Copyright (C) 2013  Michael C Richardson <mcr@xelerance.com>
  * Copyright (C) 2013  Andreas Steffen
+ * Copyright (C) 2012 Paul Wouters <paul@libreswan.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -264,8 +265,9 @@ idtoa(const struct id *id, char *dst, size_t dstlen)
     case ID_IPV4_ADDR:
     case ID_IPV6_ADDR:
 	if(isanyaddr(&id->ip_addr)) {
+	    passert(dstlen > sizeof("%any"));
 	    dst[0]='\0';
-	    strncat(dst, "%any", dstlen);
+	    strncat(dst, "%any", dstlen -1);
 	    n = strlen(dst);
 	} else {
 	    n = (int)addrtot(&id->ip_addr, 0, dst, dstlen) - 1;
