@@ -4,12 +4,13 @@
  * Copyright (C) 2001  Richard Guy Briggs  <rgb@freeswan.org>
  *                 and Michael Richardson  <mcr@freeswan.org>
  * Copyright (C) 2004  Michael Richardson  <mcr@xelerance.com>
- * 
+ * Copyright (C) 2012  Paul Wouters  <paul@libreswan.org>
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -18,7 +19,7 @@
  *
  */
 
-/* 
+/*
  * This file provides a set of #define's which may be tuned by various
  * people/configurations. It keeps all compile-time tunables in one place.
  *
@@ -88,16 +89,16 @@
  * MAIN_TABLE_WIDTH is the number of bits used for the primary index table.
  *
  */
-#ifndef IPSEC_SA_REF_MAINTABLE_IDX_WIDTH 
-# define IPSEC_SA_REF_MAINTABLE_IDX_WIDTH 4 
+#ifndef IPSEC_SA_REF_MAINTABLE_IDX_WIDTH
+# define IPSEC_SA_REF_MAINTABLE_IDX_WIDTH 4
 #endif
 
-#ifndef IPSEC_SA_REF_FREELIST_NUM_ENTRIES 
+#ifndef IPSEC_SA_REF_FREELIST_NUM_ENTRIES
 # define IPSEC_SA_REF_FREELIST_NUM_ENTRIES 256
 #endif
 
-#ifndef IPSEC_SA_REF_CODE 
-# define IPSEC_SA_REF_CODE 1 
+#ifndef IPSEC_SA_REF_CODE
+# define IPSEC_SA_REF_CODE 1
 #endif
 
 #ifdef __KERNEL__
@@ -107,41 +108,19 @@
 #endif
 
 /* always turn on IPIP mode */
-#ifndef CONFIG_KLIPS_IPIP 
+#ifndef CONFIG_KLIPS_IPIP
 #define CONFIG_KLIPS_IPIP 1
 #endif
 
 /*
  * Worry about PROC_FS stuff
  */
-#if defined(PROC_FS_2325)
 /* kernel 2.4 */
 # define IPSEC_PROC_LAST_ARG ,int *eof,void *data
 # define IPSEC_PROCFS_DEBUG_NO_STATIC
 # define IPSEC_PROC_SUBDIRS
-#else
-/* kernel <2.4 */
-# define IPSEC_PROCFS_DEBUG_NO_STATIC DEBUG_NO_STATIC
 
-# ifndef PROC_NO_DUMMY
-#  define IPSEC_PROC_LAST_ARG , int dummy
-# else
-#  define IPSEC_PROC_LAST_ARG
-# endif /* !PROC_NO_DUMMY */
-#endif /* PROC_FS_2325 */
-
-#if !defined(LINUX_KERNEL_HAS_SNPRINTF)
-/* GNU CPP specific! */
-# define snprintf(buf, len, fmt...) sprintf(buf, ##fmt)
-#endif /* !LINUX_KERNEL_HAS_SNPRINTF */
-
-#ifdef SPINLOCK
-# ifdef SPINLOCK_23
 #  include <linux/spinlock.h> /* *lock* */
-# else /* SPINLOCK_23 */
-#  include <asm/spinlock.h> /* *lock* */
-# endif /* SPINLOCK_23 */
-#endif /* SPINLOCK */
 
 #ifndef KLIPS_FIXES_DES_PARITY
 # define KLIPS_FIXES_DES_PARITY 1
@@ -163,7 +142,7 @@
 /* extra toggles for regression testing */
 #ifdef CONFIG_KLIPS_REGRESS
 
-/* 
+/*
  * should pfkey_acquire() become 100% lossy?
  *
  */
@@ -207,11 +186,6 @@ extern int sysctl_ipsec_regress_pfkey_lossage;
  */
 #define KLIPS_IMPAIRMENT_ESPIV_CBC_ATTACK 0
 
-
-/* IP_FRAGMENT_LINEARIZE is set in freeswan.h if Kernel > 2.4.4 */
-#ifndef IP_FRAGMENT_LINEARIZE
-# define IP_FRAGMENT_LINEARIZE 0
-#endif /* IP_FRAGMENT_LINEARIZE */
 #endif /* __KERNEL__ */
 
 #ifdef NEED_INET_PROTOCOL
