@@ -2,12 +2,12 @@
  * IPSEC tunneling code
  * Copyright (C) 1996, 1997  John Ioannidis.
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003  Richard Guy Briggs.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -115,13 +115,9 @@ struct ipsec_xmit_state
 	short cur_mtu;          /* copy of prv->mtu, cause prv may == NULL */
 	short mtudiff;
 	__u8 next_header;               /* protocol of the nested header */
-#ifdef NET_21
 	struct rtable *route;
-#endif /* NET_21 */
 	ip_said outgoing_said;
-#ifdef NET_21
 	int pass;
-#endif /* NET_21 */
 	uint32_t eroute_pid;
 	struct ipsec_sa ips;
 #ifdef NAT_TRAVERSAL
