@@ -3,36 +3,31 @@
  *
  * Copyright (C) 1996, 1997  John Ioannidis.
  * Copyright (C) 1998, 1999, 2000, 2001, 2002  Richard Guy Briggs.
- * 
+ * Copyright (C) 2012  Paul Wouters  <paul@libreswan.org>
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ipsec_sa.c,v 1.31 2005/11/11 04:38:56 paul Exp $
- *
  * This is the file formerly known as "ipsec_xform.h"
  *
  */
 
-#ifndef AUTOCONF_INCLUDED
+#include <linux/version.h>
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,38) && !defined(AUTOCONF_INCLUDED)
 #include <linux/config.h>
 #endif
-#include <linux/version.h>
 #include <linux/kernel.h> /* printk() */
 
 #include "openswan/ipsec_param.h"
 
-#ifdef MALLOC_SLAB
-# include <linux/slab.h> /* kmalloc() */
-#else /* MALLOC_SLAB */
-# include <linux/malloc.h> /* kmalloc() */
-#endif /* MALLOC_SLAB */
+#include <linux/slab.h> /* kmalloc() */
 #include <linux/vmalloc.h> /* vmalloc() */
 #include <linux/errno.h>  /* error codes */
 #include <linux/types.h>  /* size_t */
@@ -43,13 +38,7 @@
 #include <linux/ip.h>          /* struct iphdr */
 #include <linux/skbuff.h>
 #include <openswan.h>
-#ifdef SPINLOCK
-#ifdef SPINLOCK_23
 #include <linux/spinlock.h> /* *lock* */
-#else /* SPINLOCK_23 */
-#include <asm/spinlock.h> /* *lock* */
-#endif /* SPINLOCK_23 */
-#endif /* SPINLOCK */
 
 #include <net/ip.h>
 
@@ -77,17 +66,15 @@
 #include "openswan/ipsec_proto.h"
 #include "openswan/ipsec_alg.h"
 
-#include "ipsec_ocf.h"
+#ifdef CONFIG_KLIPS_OCF
+# include "ipsec_ocf.h"
+#endif
 
 
 #define SENDERR(_x) do { error = -(_x); goto errlab; } while (0)
 
 struct ipsec_sa *ipsec_sadb_hash[SADB_HASHMOD];
-#ifdef SPINLOCK
-spinlock_t tdb_lock = SPIN_LOCK_UNLOCKED;
-#else /* SPINLOCK */
-spinlock_t tdb_lock;
-#endif /* SPINLOCK */
+DEFINE_SPINLOCK(tdb_lock);
 
 #ifdef IPSEC_SA_RECOUNT_DEBUG
 struct ipsec_sa *ipsec_sa_raw = NULL;
@@ -106,7 +93,7 @@ typedef struct {
 /* The field where the saref will be hosted in the skb must be wide enough to
    accomodate the information it needs to store. */
 typedef struct {
-	int testSizeOf_refField : 
+	int testSizeOf_refField :
 		(IPSEC_SA_REF_HOST_FIELD_WIDTH < IPSEC_SA_REF_TABLE_IDX_WIDTH ? -1 : 1 );
 } dummy2;
 
@@ -240,7 +227,7 @@ static int
 ipsec_saref_verify_slot(IPsecSAref_t ref)
 {
 	int ref_table=IPsecSAref2table(ref);
-	
+
 	if(ipsec_sadb.refTable[ref_table] == NULL) {
 		return ipsec_SArefSubTable_alloc(ref_table);
 	}
@@ -263,7 +250,7 @@ ipsec_saref_freelist_init(void)
 	ipsec_sadb.refFreeListHead = IPSEC_SAREF_NULL;
 	ipsec_sadb.refFreeListCont = IPSEC_SAREF_FIRST;
 	ipsec_sadb.refFreeListTail = IPSEC_SAREF_NULL;
-       
+
 	return 0;
 }
 
@@ -277,7 +264,7 @@ ipsec_sadb_init(void)
 		ipsec_sadb_hash[i] = NULL;
 	}
 	/* parts above are for the old style SADB hash table */
-	
+
 
 	/* initialise SA reference table */
 
@@ -344,7 +331,7 @@ ipsec_SAref_alloc(int*error) /* pass in error var by pointer */
 		    IPsecSAref2table(SAref),
 		    IPsecSAref2entry(SAref),
 		    IPSEC_SA_REF_MAINTABLE_NUM_ENTRIES * IPSEC_SA_REF_SUBTABLE_NUM_ENTRIES);
-	
+
 	ipsec_sadb.refFreeList[ipsec_sadb.refFreeListHead] = IPSEC_SAREF_NULL;
 	ipsec_sadb.refFreeListHead++;
 	if(ipsec_sadb.refFreeListHead > ipsec_sadb.refFreeListTail) {
@@ -403,20 +390,17 @@ ipsec_sa_print(struct ipsec_sa *ips)
 	}
 	if(ips->ips_addr_s) {
 		char buf[SUBNETTOA_BUF];
-		addrtoa(((struct sockaddr_in*)(ips->ips_addr_s))->sin_addr,
-			0, buf, sizeof(buf));
+		sin_addrtot(ips->ips_addr_s, 0, buf, sizeof(buf));
 		printk(" src=%s", buf);
 	}
 	if(ips->ips_addr_d) {
 		char buf[SUBNETTOA_BUF];
-		addrtoa(((struct sockaddr_in*)(ips->ips_addr_s))->sin_addr,
-			0, buf, sizeof(buf));
+		sin_addrtot(ips->ips_addr_s, 0, buf, sizeof(buf));
 		printk(" dst=%s", buf);
 	}
 	if(ips->ips_addr_p) {
 		char buf[SUBNETTOA_BUF];
-		addrtoa(((struct sockaddr_in*)(ips->ips_addr_p))->sin_addr,
-			0, buf, sizeof(buf));
+		sin_addrtot(ips->ips_addr_p, 0, buf, sizeof(buf));
 		printk(" proxy=%s", buf);
 	}
 	if(ips->ips_key_bits_a) {
@@ -478,13 +462,13 @@ ipsec_sa_untern(struct ipsec_sa *ips)
 			    "ref=%u -> %p but untern'ing %p\n", ref,
 			    IPsecSAref2SA(ref), ips);
 	}
-		
+
 }
 
 int
 ipsec_sa_intern(struct ipsec_sa *ips)
 {
-	int error;
+	int error = 0;
 	IPsecSAref_t ref = ips->ips_ref;
 
 	if(ref == IPSEC_SAREF_NULL) {
@@ -548,7 +532,7 @@ ipsec_sa_getbyid(ip_said *said, int type)
 	}
 
 	hashval = IPS_HASH(said);
-	
+
 	sa_len = KLIPS_SATOT(debug_xform, said, 0, sa, sizeof(sa));
 	KLIPS_PRINT(debug_xform,
 		    "ipsec_sa_getbyid: "
@@ -567,13 +551,13 @@ ipsec_sa_getbyid(ip_said *said, int type)
 
 	for (; ips; ips = ips->ips_hnext) {
 		if ((ips->ips_said.spi == said->spi) &&
-		    (ips->ips_said.dst.u.v4.sin_addr.s_addr == said->dst.u.v4.sin_addr.s_addr) &&
+		    (ip_address_cmp(&ips->ips_said.dst, &said->dst) == 0) &&
 		    (ips->ips_said.proto == said->proto)) {
 			ipsec_sa_get(ips, type);
 			return ips;
 		}
 	}
-	
+
 	KLIPS_PRINT(debug_xform,
 		    "ipsec_sa_getbyid: "
 		    "no entry in linked list for hash=%d of SA:%s.\n",
@@ -722,10 +706,10 @@ ipsec_sa_add(struct ipsec_sa *ips)
 
 	ipsec_sa_get(ips, IPSEC_REFSAADD);
 	spin_lock_bh(&tdb_lock);
-	
+
 	ips->ips_hnext = ipsec_sadb_hash[hashval];
 	ipsec_sadb_hash[hashval] = ips;
-	
+
 	spin_unlock_bh(&tdb_lock);
 
 	return error;
@@ -757,7 +741,7 @@ void ipsec_sa_rm(struct ipsec_sa *ips)
 	if(ipsec_sadb_hash[hashval] == NULL) {
 		return;
 	}
-	
+
 	if (ips == ipsec_sadb_hash[hashval]) {
 		ipsec_sadb_hash[hashval] = ipsec_sadb_hash[hashval]->ips_hnext;
 		ips->ips_hnext = NULL;
@@ -784,12 +768,12 @@ void ipsec_sa_rm(struct ipsec_sa *ips)
 		}
 	}
 }
-	
-	
+
+
 #if 0
 /*
  * The ipsec_sa table better be locked before it is handed in,
- * or races might happen. 
+ * or races might happen.
  *
  * this routine assumes the SA has a refcount==0, and we free it.
  * we also assume that the pointers are already cleaned up.
@@ -815,10 +799,10 @@ ipsec_sa_del(struct ipsec_sa *ips)
 		ips->ips_next=NULL;
 		ipsec_sa_put(in);
 	}
-	
+
 	sa_len = KLIPS_SATOT(debug_xform, &ips->ips_said, 0, sa, sizeof(sa));
 	hashval = IPS_HASH(&ips->ips_said);
-	
+
 	KLIPS_PRINT(debug_xform,
 		    "klips_debug:ipsec_sa_del: "
 		    "deleting SA:%s (ref=%u), hashval=%d.\n",
@@ -838,7 +822,7 @@ ipsec_sa_del(struct ipsec_sa *ips)
 			    sa_len ? sa : " (error)");
 		return -ENOENT;
 	}
-	
+
 	if (ips == ipsec_sadb_hash[hashval]) {
 		ipsec_sadb_hash[hashval] = ipsec_sadb_hash[hashval]->ips_hnext;
 		ips->ips_hnext = NULL;
@@ -863,7 +847,7 @@ ipsec_sa_del(struct ipsec_sa *ips)
 			}
 		}
 	}
-	
+
 	KLIPS_PRINT(debug_xform,
 		    "klips_debug:ipsec_sa_del: "
 		    "no entries in linked list for hash=%d of SA:%s.\n",
@@ -873,7 +857,7 @@ ipsec_sa_del(struct ipsec_sa *ips)
 }
 #endif
 
-int 
+int
 ipsec_sadb_cleanup(__u8 proto)
 {
 	unsigned i;
@@ -882,7 +866,7 @@ ipsec_sadb_cleanup(__u8 proto)
 	/* struct ipsec_sa *ipsnext, **ipsprev; */
 	/* char sa[SATOT_BUF]; */
 	/* size_t sa_len; */
-	 
+
 
 	KLIPS_PRINT(debug_xform,
 		    "klips_debug:ipsec_sadb_cleanup: "
@@ -893,7 +877,7 @@ ipsec_sadb_cleanup(__u8 proto)
 
 	for (i = 0; i < SADB_HASHMOD; i++) {
 		ips = ipsec_sadb_hash[i];
-		
+
 		while(ips) {
 			ipsec_sadb_hash[i]=ips->ips_hnext;
 			ips->ips_hnext=NULL;
@@ -944,7 +928,7 @@ ipsec_sadb_cleanup(__u8 proto)
 	return(error);
 }
 
-int 
+int
 ipsec_sadb_free(void)
 {
 	int error = 0;
@@ -1000,7 +984,7 @@ ipsec_sa_wipe(struct ipsec_sa *ips)
 
 #if IPSEC_SA_REF_CODE
 	/* remove me from the SArefTable */
-	if(debug_xform) 
+	if(debug_xform)
 	{
 		char sa[SATOT_BUF];
 		size_t sa_len;
@@ -1031,7 +1015,7 @@ ipsec_sa_wipe(struct ipsec_sa *ips)
 		if(ref_table < IPSEC_SA_REF_SUBTABLE_NUM_ENTRIES) {
 			subtable = ipsec_sadb.refTable[ref_table];
 			if(subtable!=NULL && subtable->entry[ref_entry] == ips) {
-			
+
 				subtable->entry[ref_entry] = NULL;
 			}
 		}
@@ -1077,7 +1061,7 @@ ipsec_sa_wipe(struct ipsec_sa *ips)
 		if (ips->ips_alg_enc &&
 		    ips->ips_alg_enc->ixt_e_destroy_key)
 		{
-			ips->ips_alg_enc->ixt_e_destroy_key(ips->ips_alg_enc, 
+			ips->ips_alg_enc->ixt_e_destroy_key(ips->ips_alg_enc,
 							    ips->ips_key_e);
 		} else
 #endif
@@ -1106,7 +1090,7 @@ ipsec_sa_wipe(struct ipsec_sa *ips)
 		kfree(ips->ips_ident_s.data);
         }
 	ips->ips_ident_s.data = NULL;
-	
+
 	if(ips->ips_ident_d.data != NULL) {
 		memset((caddr_t)(ips->ips_ident_d.data),
                        0,
@@ -1153,6 +1137,10 @@ ipsec_sa_wipe(struct ipsec_sa *ips)
 		}
 	}
 #endif
+	if(ips->ips_out != NULL) {
+		ipsec_dev_put(ips->ips_out);
+		ips->ips_out = NULL;
+	}
 
 	memset((caddr_t)ips, 0, sizeof(*ips));
 	kfree(ips);
@@ -1168,8 +1156,10 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
         int error = 0;
         char sa[SATOT_BUF];
 	size_t sa_len;
+#ifdef CONFIG_KLIPS_DEBUG
 	char ipaddr_txt[ADDRTOA_BUF];
 	char ipaddr2_txt[ADDRTOA_BUF];
+#endif
 #if defined (CONFIG_KLIPS_AUTH_HMAC_MD5) || defined (CONFIG_KLIPS_AUTH_HMAC_SHA1)
 	unsigned char kb[AHMD596_BLKLEN];
 	int i;
@@ -1193,22 +1183,20 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 		    "ipsec_sa_init: "
 		    "calling init routine of %s%s%s\n",
 		    IPS_XFORM_NAME(ipsp));
-	
+
 	switch(ipsp->ips_said.proto) {
 #ifdef CONFIG_KLIPS_IPIP
 	case IPPROTO_IPIP: {
 		ipsp->ips_xformfuncs = ipip_xform_funcs;
-		addrtoa(((struct sockaddr_in*)(ipsp->ips_addr_s))->sin_addr,
-			0,
-			ipaddr_txt, sizeof(ipaddr_txt));
-		addrtoa(((struct sockaddr_in*)(ipsp->ips_addr_d))->sin_addr,
-			0,
-			ipaddr2_txt, sizeof(ipaddr_txt));
+#ifdef CONFIG_KLIPS_DEBUG
+		sin_addrtot(ipsp->ips_addr_s, 0, ipaddr_txt, sizeof(ipaddr_txt));
+		sin_addrtot(ipsp->ips_addr_d, 0, ipaddr2_txt, sizeof(ipaddr2_txt));
 		KLIPS_PRINT(debug_pfkey,
 			    "ipsec_sa_init: "
 			    "(pfkey defined) IPIP ipsec_sa set for %s->%s.\n",
 			    ipaddr_txt,
 			    ipaddr2_txt);
+#endif
 	}
 	break;
 #endif /* !CONFIG_KLIPS_IPIP */
@@ -1216,12 +1204,13 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 #ifdef CONFIG_KLIPS_AH
 	case IPPROTO_AH:
 
+		ipsp->ips_xformfuncs = ah_xform_funcs;
+
 #ifdef CONFIG_KLIPS_OCF
 		if (ipsec_ocf_sa_init(ipsp, ipsp->ips_authalg, 0))
 		    break;
 #endif
 
-		ipsp->ips_xformfuncs = ah_xform_funcs;
 		switch(ipsp->ips_authalg) {
 # ifdef CONFIG_KLIPS_AUTH_HMAC_MD5
 		case AH_MD5: {
@@ -1229,7 +1218,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 			unsigned int aks;
 			MD5_CTX *ictx;
 			MD5_CTX *octx;
-			
+
 			if(ipsp->ips_key_bits_a != (AHMD596_KLEN * 8)) {
 				KLIPS_PRINT(debug_pfkey,
 					    "ipsec_sa_init: "
@@ -1237,7 +1226,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 					    ipsp->ips_key_bits_a, AHMD596_KLEN * 8);
 				SENDERR(EINVAL);
 			}
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
@@ -1247,13 +1236,13 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 				    ntohl(*(((__u32 *)ipsp->ips_key_a)+2)),
 				    ntohl(*(((__u32 *)ipsp->ips_key_a)+3)));
 #  endif /* KLIPS_DIVULGE_HMAC_KEY */
-			
+
 			ipsp->ips_auth_bits = AHMD596_ALEN * 8;
-			
+
 			/* save the pointer to the key material */
 			akp = ipsp->ips_key_a;
 			aks = ipsp->ips_key_a_size;
-			
+
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 			           "ipsec_sa_init: "
 			           "allocating %lu bytes for md5_ctx.\n",
@@ -1283,7 +1272,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 			octx = &(((struct md5_ctx*)(ipsp->ips_key_a))->octx);
 			osMD5Init(octx);
 			osMD5Update(octx, kb, AHMD596_BLKLEN);
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
@@ -1297,7 +1286,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 				    ((__u32*)octx)[2],
 				    ((__u32*)octx)[3] );
 #  endif /* KLIPS_DIVULGE_HMAC_KEY */
-			
+
 			/* zero key buffer -- paranoid */
 			memset(akp, 0, aks);
 			kfree(akp);
@@ -1310,7 +1299,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 			unsigned int aks;
 			SHA1_CTX *ictx;
 			SHA1_CTX *octx;
-			
+
 			if(ipsp->ips_key_bits_a != (AHSHA196_KLEN * 8)) {
 				KLIPS_PRINT(debug_pfkey,
 					    "ipsec_sa_init: "
@@ -1318,7 +1307,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 					    ipsp->ips_key_bits_a, AHSHA196_KLEN * 8);
 				SENDERR(EINVAL);
 			}
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
@@ -1328,13 +1317,13 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 				    ntohl(*(((__u32 *)ipsp->ips_key_a)+2)),
 				    ntohl(*(((__u32 *)ipsp->ips_key_a)+3)));
 #  endif /* KLIPS_DIVULGE_HMAC_KEY */
-			
+
 			ipsp->ips_auth_bits = AHSHA196_ALEN * 8;
-			
+
 			/* save the pointer to the key material */
 			akp = ipsp->ips_key_a;
 			aks = ipsp->ips_key_a_size;
-			
+
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 			            "ipsec_sa_init: "
 			            "allocating %lu bytes for sha1_ctx.\n",
@@ -1364,11 +1353,11 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 			octx = &(((struct sha1_ctx*)(ipsp->ips_key_a))->octx);
 			SHA1Init(octx);
 			SHA1Update(octx, kb, AHSHA196_BLKLEN);
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
-				    "SHA1 ictx=0x%08x %08x %08x %08x octx=0x%08x %08x %08x %08x\n", 
+				    "SHA1 ictx=0x%08x %08x %08x %08x octx=0x%08x %08x %08x %08x\n",
 				    ((__u32*)ictx)[0],
 				    ((__u32*)ictx)[1],
 				    ((__u32*)ictx)[2],
@@ -1411,9 +1400,9 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 		error = ipsec_alg_auth_key_create(ipsp);
 		if ((error < 0) && (error != -EPROTO))
 			SENDERR(-error);
-		
+
 		if (error == -EPROTO) {
-			/* perform manual key generation, 
+			/* perform manual key generation,
 			   ignore this particular error */
 			error = 0;
 #endif /* CONFIG_KLIPS_ALG */
@@ -1436,7 +1425,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 					    AHMD596_KLEN * 8);
 				SENDERR(EINVAL);
 			}
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
@@ -1447,11 +1436,11 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 				    ntohl(*(((__u32 *)(ipsp->ips_key_a))+3)));
 #  endif /* KLIPS_DIVULGE_HMAC_KEY */
 			ipsp->ips_auth_bits = AHMD596_ALEN * 8;
-			
+
 			/* save the pointer to the key material */
 			akp = ipsp->ips_key_a;
 			aks = ipsp->ips_key_a_size;
-			
+
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 			            "ipsec_sa_init: "
 			            "allocating %lu bytes for md5_ctx.\n",
@@ -1481,7 +1470,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 			octx = &(((struct md5_ctx*)(ipsp->ips_key_a))->octx);
 			osMD5Init(octx);
 			osMD5Update(octx, kb, AHMD596_BLKLEN);
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
@@ -1514,7 +1503,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 					    AHSHA196_KLEN * 8);
 				SENDERR(EINVAL);
 			}
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
@@ -1525,7 +1514,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 				    ntohl(*(((__u32 *)ipsp->ips_key_a)+3)));
 #  endif /* KLIPS_DIVULGE_HMAC_KEY */
 			ipsp->ips_auth_bits = AHSHA196_ALEN * 8;
-			
+
 			/* save the pointer to the key material */
 			akp = ipsp->ips_key_a;
 			aks = ipsp->ips_key_a_size;
@@ -1559,7 +1548,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 			octx = &((struct sha1_ctx*)(ipsp->ips_key_a))->octx;
 			SHA1Init(octx);
 			SHA1Update(octx, kb, AHSHA196_BLKLEN);
-			
+
 #  if KLIPS_DIVULGE_HMAC_KEY
 			KLIPS_PRINT(debug_pfkey && sysctl_ipsec_debug_verbose,
 				    "ipsec_sa_init: "
@@ -1596,11 +1585,11 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 
 		/* Create IV */
 		if (ipsp->ips_iv_size) {
-			if ((ipsp->ips_iv = 
+			if ((ipsp->ips_iv =
 			     (caddr_t)kmalloc(ipsp->ips_iv_size, GFP_ATOMIC)) == NULL) {
 				SENDERR(ENOMEM);
 			}
-			prng_bytes(&ipsec_prng, (char *)ipsp->ips_iv, 
+			prng_bytes(&ipsec_prng, (char *)ipsp->ips_iv,
 				   ipsp->ips_iv_size);
 			ipsp->ips_iv_bits = ipsp->ips_iv_size * 8;
 		}
@@ -1609,7 +1598,19 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 #endif /* !CONFIG_KLIPS_ESP */
 #ifdef CONFIG_KLIPS_IPCOMP
 	case IPPROTO_COMP:
+
 		ipsp->ips_xformfuncs = ipcomp_xform_funcs;
+
+		ipsp->ips_comp_adapt_tries = 0;
+		ipsp->ips_comp_adapt_skip = 0;
+		ipsp->ips_comp_ratio_cbytes = 0;
+		ipsp->ips_comp_ratio_dbytes = 0;
+
+#ifdef CONFIG_KLIPS_OCF
+		if (ipsec_ocf_comp_sa_init(ipsp, ipsp->ips_encalg))
+			break;
+#endif
+
 		ipsp->ips_comp_adapt_tries = 0;
 		ipsp->ips_comp_adapt_skip = 0;
 		ipsp->ips_comp_ratio_cbytes = 0;
@@ -1622,7 +1623,7 @@ int ipsec_sa_init(struct ipsec_sa *ipsp)
 		       ipsp->ips_said.proto);
 		SENDERR(EINVAL);
 	}
-	
+
 errlab:
 	return(error);
 }
