@@ -1,6 +1,7 @@
 /*
  * @(#) RFC2367 PF_KEYv2 Key management API domain socket I/F
  * Copyright (C) 1999, 2000, 2001  Richard Guy Briggs.
+ * Copyright (C) 2012  Paul Wouters <paul@libreswan.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -44,11 +45,7 @@
 #include <linux/net.h> /* struct socket */
 #include <linux/in.h>
 #include <linux/fs.h>
-#ifdef MALLOC_SLAB
-# include <linux/slab.h> /* kmalloc() */
-#else /* MALLOC_SLAB */
-# include <linux/malloc.h> /* kmalloc() */
-#endif /* MALLOC_SLAB */
+#include <linux/slab.h> /* kmalloc() */
 #ifdef CONFIG_X86
 # include <asm/segment.h>
 #endif
@@ -210,6 +207,50 @@ static __inline__ void pfkey_unlock_sock_list(void)
 }
 #endif
 
+/*****
+Grabbed Mr. Viro's proc_subtree backport code for older kernels
+commit 8ce584c7416d8a85a6f3edc17d1cddefe331e87e
+Author: Al Viro <v...@zeniv.linux.org.uk>
+Date:   Sat Mar 30 20:13:46 2013 -0400
+****/
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0)
+#ifdef CONFIG_PROC_FS
+
+static void backport_proc_subdir_remove(struct proc_dir_entry *dir)
+{
+       struct proc_dir_entry *pe, *tmp;
+       pe = dir->subdir;
+       while (pe) {
+               tmp = pe->next;
+               backport_proc_subdir_remove(pe);
+               remove_proc_entry(pe->name, dir);
+               pe = tmp;
+       }
+};
+
+int remove_proc_subtree(const char *name, struct proc_dir_entry *parent)
+{
+       struct proc_dir_entry *pe, *tmp;
+
+       if (!parent)
+               goto out;
+
+       pe = parent->subdir;
+       while (pe) {
+               tmp = pe->next;
+               backport_proc_subdir_remove(pe);
+               remove_proc_entry(pe->name, parent);
+               pe = tmp;
+       }
+
+out:
+       remove_proc_entry(name, parent);
+
+       return 0;
+}
+#endif /* CONFIG_PROC_FS */
+#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(3,10,0) */
+
 int
 pfkey_list_remove_socket(struct socket *socketp, struct socket_list **sockets)
 {
@@ -512,10 +553,6 @@ pfkey_destroy_socket(struct sock *sk)
 			}
 			printk(" len:%d", skb->len);
 			printk(" csum:%d", skb->csum);
-#ifndef NETDEV_23
-			printk(" used:%d", skb->used);
-			printk(" is_clone:%d", skb->is_clone);
-#endif /* NETDEV_23 */
 			printk(" cloned:%d", skb->cloned);
 			printk(" pkt_type:%d", skb->pkt_type);
 			printk(" ip_summed:%d", skb->ip_summed);
@@ -764,11 +801,7 @@ pfkey_create(struct socket *sock, int protocol)
 }
 
 DEBUG_NO_STATIC int
-#ifdef NETDEV_23
 pfkey_release(struct socket *sock)
-#else /* NETDEV_23 */
-pfkey_release(struct socket *sock, struct socket *peersock)
-#endif /* NETDEV_23 */
 {
 	struct sock *sk;
 	int i;
@@ -1133,14 +1166,8 @@ pfkey_recvmsg(struct socket *sock
 }
 
 #ifdef CONFIG_PROC_FS
-#ifndef PROC_FS_2325
-DEBUG_NO_STATIC
-#endif /* PROC_FS_2325 */
 int
 pfkey_get_info(char *buffer, char **start, off_t offset, int length
-#ifndef  PROC_NO_DUMMY
-, int dummy
-#endif /* !PROC_NO_DUMMY */
 #ifdef  PROC_EOF_DATA
 , int *eof
 , void *data
@@ -1232,14 +1259,8 @@ pfkey_get_info(char *buffer, char **start, off_t offset, int length
 	return len - (offset - begin);
 }
 
-#ifndef PROC_FS_2325
-DEBUG_NO_STATIC
-#endif /* PROC_FS_2325 */
 int
 pfkey_supported_get_info(char *buffer, char **start, off_t offset, int length
-#ifndef  PROC_NO_DUMMY
-, int dummy
-#endif /* !PROC_NO_DUMMY */
 #ifdef  PROC_EOF_DATA
 , int *eof
 , void *data
@@ -1296,14 +1317,8 @@ pfkey_supported_get_info(char *buffer, char **start, off_t offset, int length
 	return len - (offset - begin);
 }
 
-#ifndef PROC_FS_2325
-DEBUG_NO_STATIC
-#endif /* PROC_FS_2325 */
 int
 pfkey_registered_get_info(char *buffer, char **start, off_t offset, int length
-#ifndef  PROC_NO_DUMMY
-, int dummy
-#endif /* !PROC_NO_DUMMY */
 #ifdef  PROC_EOF_DATA
 , int *eof
 , void *data
@@ -1352,32 +1367,6 @@ pfkey_registered_get_info(char *buffer, char **start, off_t offset, int length
 	return len - (offset - begin);
 }
 
-#ifndef PROC_FS_2325
-struct proc_dir_entry proc_net_pfkey =
-{
-	0,
-	6, "pf_key",
-	S_IFREG | S_IRUGO, 1, 0, 0,
-	0, &proc_net_inode_operations,
-	pfkey_get_info
-};
-struct proc_dir_entry proc_net_pfkey_supported =
-{
-	0,
-	16, "pf_key_supported",
-	S_IFREG | S_IRUGO, 1, 0, 0,
-	0, &proc_net_inode_operations,
-	pfkey_supported_get_info
-};
-struct proc_dir_entry proc_net_pfkey_registered =
-{
-	0,
-	17, "pf_key_registered",
-	S_IFREG | S_IRUGO, 1, 0, 0,
-	0, &proc_net_inode_operations,
-	pfkey_registered_get_info
-};
-#endif /* !PROC_FS_2325 */
 #endif /* CONFIG_PROC_FS */
 
 DEBUG_NO_STATIC int
@@ -1517,7 +1506,6 @@ pfkey_init(void)
                 entry->read_proc = pfkey_registered_get_info;
         }
 #endif /* CONFIG_PROC_FS */
-
 	return error;
 }
 
@@ -1542,15 +1530,9 @@ pfkey_cleanup(void)
 	error |= supported_remove_all(K_SADB_X_SATYPE_IPIP);
 
 #ifdef CONFIG_PROC_FS
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0)
         remove_proc_subtree("pf_key",            init_net.proc_net);
         remove_proc_subtree("pf_key_supported",  init_net.proc_net);
         remove_proc_subtree("pf_key_registered", init_net.proc_net);
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(3,9,0) */
-        proc_net_remove (&init_net, "pf_key");
-        proc_net_remove (&init_net, "pf_key_supported");
-        proc_net_remove (&init_net, "pf_key_registered");
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(3,9,0) */
 #endif /* CONFIG_PROC_FS */
 
 	/* other module unloading cleanup happens here */
