@@ -767,6 +767,13 @@ main_inI1_outR1(struct msg_digest *md)
     {
 	pb_stream pre_sa_pbs = sa_pd->pbs;
 	lset_t policy = preparse_isakmp_sa_body(&pre_sa_pbs);
+	/*
+	 * If there is XAUTH VID, copy it to policies.
+	 */
+	if (md->quirks.xauth_vid == TRUE)
+	{
+	  policy |= POLICY_XAUTH;
+	}
 	/* See if a wildcarded connection can be found.
 	 * We cannot pick the right connection, so we're making a guess.
 	 * All Road Warrior connections are fair game:
@@ -2233,7 +2240,9 @@ main_inI3_outR3_tail(struct msg_digest *md
 
     /* It seems as per Cisco implementation, XAUTH and MODECFG
      * are not supposed to be performed again during rekey */
-    if(st->st_connection->newest_isakmp_sa != SOS_NOBODY &&
+
+    if( st->st_connection->remotepeertype == CISCO &&
+	st->st_connection->newest_isakmp_sa != SOS_NOBODY &&
         st->st_connection->spd.this.xauth_client) {
            DBG(DBG_CONTROL, DBG_log("Skipping XAUTH for rekey for Cisco Peer compatibility."));
            st->hidden_variables.st_xauth_client_done = TRUE;
@@ -2306,7 +2315,8 @@ main_inR3_tail(struct msg_digest *md
 
     /* It seems as per Cisco implementation, XAUTH and MODECFG
      * are not supposed to be performed again during rekey */
-    if(st->st_connection->newest_isakmp_sa != SOS_NOBODY &&
+    if( st->st_connection->remotepeertype == CISCO &&
+	st->st_connection->newest_isakmp_sa != SOS_NOBODY &&
         st->st_connection->spd.this.xauth_client) {
            DBG(DBG_CONTROL, DBG_log("Skipping XAUTH for rekey for Cisco Peer compatibility."));
            st->hidden_variables.st_xauth_client_done = TRUE;
