@@ -81,20 +81,18 @@ struct state_v2_microcode {
     enum state_kind state, next_state;
     enum isakmp_xchg_types recv_type;
     lset_t flags;
-    lset_t req_payloads;	/* required payloads (allows just one) */
-    lset_t opt_payloads;	/* optional payloads (any mumber) */
+    lset_t req_clear_payloads;  /* required unencrypted payloads (allows just one) for received packet */
+    lset_t opt_clear_payloads;  /* optional unencrypted payloads (none or one) for received packet */
+    lset_t req_enc_payloads;  /* required encrypted payloads (allows just one) for received packet */
+    lset_t opt_enc_payloads;  /* optional encrypted payloads (none or one) for received packet */
     enum event_type timeout_event;
     state_transition_fn *processor;
 };
 
 enum smf2_flags {
     SMF2_INITIATOR = LELEM(1),
-    SMF2_RESPONDER = 0,
-
-    SMF2_STATENEEDED=LELEM(2),
-    SMF2_NEWSTATE  = 0,
-
-    SMF2_REPLY     = LELEM(3),
+    SMF2_STATENEEDED = LELEM(2),
+    SMF2_REPLY = LELEM(3),
 };
 
 /*
@@ -133,18 +131,26 @@ enum smf2_flags {
  *
  */
 
-/* it is not clear how the flags will be used yet, if at all */
+#define PT(n) ISAKMP_NEXT_v2 ## n
+#define P(n) LELEM(PT(n) - ISAKMP_v2PAYLOAD_TYPE_BASE)
+static const lset_t everywhere_payloads = P(N) | P(V); /* can appear in any packet */
+static const lset_t repeatable_payloads = P(N) | P(D) | P(CP) | P(V);  /* if one can appear, many can appear */
 
-static const struct state_v2_microcode state_microcode_table[] = {
+/* microcode to parent first initiator state: not associated with an input packet */
+const struct state_v2_microcode ikev2_parent_firststate_microcode =
     { .state      = STATE_UNDEFINED,
       .next_state = STATE_PARENT_I1,
       .flags      = SMF2_INITIATOR,
       .processor  = NULL,
-    },
+};
 
+/* microcode for input packet processing */
+static const struct state_v2_microcode v2_state_microcode_table[] = {
     { .state      = STATE_PARENT_I1,
       .next_state = STATE_PARENT_I2,
       .flags = SMF2_INITIATOR|SMF2_STATENEEDED|SMF2_REPLY,
+      .req_clear_payloads = P(SA) | P(KE) | P(Nr),
+      .opt_clear_payloads = P(CERTREQ),
       .processor  = ikev2parent_inR1outI2,
       .recv_type  = ISAKMP_v2_SA_INIT,
     },
@@ -152,6 +158,9 @@ static const struct state_v2_microcode state_microcode_table[] = {
     { .state      = STATE_PARENT_I2,
       .next_state = STATE_PARENT_I3,
       .flags = SMF2_INITIATOR|SMF2_STATENEEDED,
+      .req_clear_payloads = P(E),
+      .req_enc_payloads = P(IDr) | P(AUTH) | P(SA) | P(TSi) | P(TSr),
+      .opt_enc_payloads = P(CERT),
       .processor  = ikev2parent_inR2,
       .recv_type  = ISAKMP_v2_AUTH,
       .timeout_event = EVENT_SA_REPLACE,
@@ -159,14 +168,18 @@ static const struct state_v2_microcode state_microcode_table[] = {
 
     { .state      = STATE_UNDEFINED,
       .next_state = STATE_PARENT_R1,
-      .flags = SMF2_RESPONDER|SMF2_NEWSTATE|SMF2_REPLY,
+      .flags =  /* not SMF2_INITIATOR, not SMF2_STATENEEDED */ SMF2_REPLY,
+      .req_clear_payloads = P(SA) | P(KE) | P(Ni),
       .processor  = ikev2parent_inI1outR1,
       .recv_type  = ISAKMP_v2_SA_INIT,
     },
 
     { .state      = STATE_PARENT_R1,
       .next_state = STATE_PARENT_R2,
-      .flags = SMF2_RESPONDER|SMF2_STATENEEDED|SMF2_REPLY,
+      .flags =  /* not SMF2_INITIATOR */ SMF2_STATENEEDED | SMF2_REPLY,
+      .req_clear_payloads = P(E),
+      .req_enc_payloads = P(IDi) | P(AUTH) | P(SA) | P(TSi) | P(TSr),
+      .opt_enc_payloads = P(CERT) | P(CERTREQ) | P(IDr),
       .processor  = ikev2parent_inI2outR2,
       .recv_type  = ISAKMP_v2_AUTH,
       .timeout_event = EVENT_SA_REPLACE,
@@ -176,6 +189,8 @@ static const struct state_v2_microcode state_microcode_table[] = {
     { .state      = STATE_PARENT_I2,
       .next_state = STATE_PARENT_I2,
       .flags      = SMF2_STATENEEDED,
+      .req_clear_payloads = P(E),
+      .opt_enc_payloads = P(N) | P(D) | P(CP),
       .processor  = process_informational_ikev2,
       .recv_type  = ISAKMP_v2_INFORMATIONAL,
     },
@@ -185,6 +200,8 @@ static const struct state_v2_microcode state_microcode_table[] = {
     { .state      = STATE_PARENT_R1,
       .next_state = STATE_PARENT_R1,
       .flags      = SMF2_STATENEEDED,
+      .req_clear_payloads = P(E),
+      .opt_enc_payloads = P(N) | P(D) | P(CP),
       .processor  = process_informational_ikev2,
       .recv_type  = ISAKMP_v2_INFORMATIONAL,
     },
@@ -193,6 +210,8 @@ static const struct state_v2_microcode state_microcode_table[] = {
     { .state      = STATE_PARENT_I3,
       .next_state = STATE_PARENT_I3,
       .flags      = SMF2_STATENEEDED,
+      .req_clear_payloads = P(E),
+      .opt_enc_payloads = P(N) | P(D) | P(CP),
       .processor  = process_informational_ikev2,
       .recv_type  = ISAKMP_v2_INFORMATIONAL,
     },
@@ -201,6 +220,8 @@ static const struct state_v2_microcode state_microcode_table[] = {
     { .state      = STATE_PARENT_R2,
       .next_state = STATE_PARENT_R2,
       .flags      = SMF2_STATENEEDED,
+      .req_clear_payloads = P(E),
+      .opt_enc_payloads = P(N) | P(D) | P(CP),
       .processor  = process_informational_ikev2,
       .recv_type  = ISAKMP_v2_INFORMATIONAL,
     },
@@ -209,6 +230,8 @@ static const struct state_v2_microcode state_microcode_table[] = {
     { .state      = STATE_IKESA_DEL,
       .next_state = STATE_IKESA_DEL,
       .flags      = SMF2_STATENEEDED,
+      .req_clear_payloads = P(E),
+      .opt_enc_payloads = P(N) | P(D) | P(CP),
       .processor  = process_informational_ikev2,
       .recv_type  = ISAKMP_v2_INFORMATIONAL,
     },
@@ -218,105 +241,143 @@ static const struct state_v2_microcode state_microcode_table[] = {
     { .state      = STATE_IKEv2_ROOF }
 };
 
-
-const struct state_v2_microcode *ikev2_parent_firststate()
-{
-    return &state_microcode_table[0];
-}
-
+#undef P
+#undef PT
 
 /*
  * split up an incoming message into payloads
  */
-stf_status
+static stf_status
 ikev2_process_payloads(struct msg_digest *md,
 			    pb_stream    *in_pbs,
-			    unsigned int from_state,
-			    unsigned int np)
+          unsigned int np,
+          lset_t req_payloads,
+          lset_t opt_payloads)
 {
     struct payload_digest *pd = md->digest_roof;
-    struct state *st = md->st;
-
-    /* lset_t needed = smc->req_payloads; */
-
-    /* zero out the digest descriptors -- might nuke [v2E] digest! */
+    lset_t seen = LEMPTY;
+    /* ??? zero out the digest descriptors -- might nuke ISAKMP_NEXT_v2E digest! */
 
     while (np != ISAKMP_NEXT_NONE)
     {
-	struct_desc *sd = np < ISAKMP_NEXT_ROOF? payload_descs[np] : NULL;
-	int thisp = np;
-	bool unknown_payload = FALSE;
+  struct_desc *sd = payload_desc(np);
 
 	memset(pd, 0, sizeof(*pd));
 
 	if (pd == &md->digest[PAYLIMIT])
 	{
 	    loglog(RC_LOG_SERIOUS, "more than %d payloads in message; ignored", PAYLIMIT);
-	    SEND_NOTIFICATION(PAYLOAD_MALFORMED);
-	    return STF_FAIL;
+      return STF_FAIL + v2N_INVALID_SYNTAX;
 	}
 
-	if (sd == NULL)
-	{
-	    unknown_payload = TRUE;
-	    sd = &ikev2_generic_desc;
+
+  memset(pd, 0, sizeof(*pd));     /* ??? is this needed? */
+
+
+  if (sd == NULL || np < ISAKMP_v2PAYLOAD_TYPE_BASE) {
+    /* This payload is unknown to us.
+     * RFCs 4306 and 5996 2.5 say that if the payload
+     * has the Critical Bit, we should be upset
+     * but if it does not, we should just ignore it.
+     */
+    if (!in_struct(&pd->payload, &ikev2_generic_desc, in_pbs, &pd->pbs)) {
+      loglog(RC_LOG_SERIOUS, "malformed payload in packet");
+      return STF_FAIL + v2N_INVALID_SYNTAX;
+    }
+    if (pd->payload.v2gen.isag_critical & ISAKMP_PAYLOAD_CRITICAL) {
+      /* It was critical.
+       * See RFC 5996 1.5 "Version Numbers and Forward Compatibility"
+       * ??? we are supposed to send the offending np byte back in the
+       * notify payload.
+       */
+      loglog(RC_LOG_SERIOUS,
+             "critical payload (%s) was not understood. Message dropped.",
+             enum_show(&payload_names_ikev2, np));
+      return STF_FAIL + v2N_UNSUPPORTED_CRITICAL_PAYLOAD;
+    }
+    loglog(RC_COMMENT, "non-critical payload ignored because it contains an unknown or"
+           " unexpected payload type (%s) at the outermost level",
+           enum_show(&payload_names_ikev2, np));
+    np = pd->payload.generic.isag_np;
+    continue;
 	}
 
-	/* why to process an unknown payload*/
-	/* critical bit in RFC 4306/5996 is just 1 bit not a byte*/
-	/* As per RFC other 7 bits are RESERVED and should be ignored*/
-	if(unknown_payload) {
-	    if(pd->payload.v2gen.isag_critical & ISAKMP_PAYLOAD_CRITICAL) {
-		/* it was critical */
-		loglog(RC_LOG_SERIOUS, "critical payload (%s) was not understood. Message dropped."
-		       , enum_show(&payload_names, thisp));
-		return STF_FATAL;
-	    }
-	    loglog(RC_COMMENT, "non-critical payload ignored because it contains an unknown or"
-		   " unexpected payload type (%s) at the outermost level"
-		   , enum_show(&payload_names, thisp));
+  passert(np - ISAKMP_v2PAYLOAD_TYPE_BASE < LELEM_ROOF);
+
+  {
+    lset_t s = LELEM(np - ISAKMP_v2PAYLOAD_TYPE_BASE);
+
+    if (s & seen & ~repeatable_payloads) {
+      /* improperly repeated payload */
+      loglog(RC_LOG_SERIOUS,
+             "payload (%s) unexpectedly repeated. Message dropped.",
+             enum_show(&payload_names_ikev2, np));
+      return STF_FAIL + v2N_INVALID_SYNTAX;
+    }
+    if ((s & (req_payloads | opt_payloads | everywhere_payloads)) == LEMPTY) {
+      /* unexpected payload */
+      loglog(RC_LOG_SERIOUS,
+             "payload (%s) unexpected. Message dropped.",
+             enum_show(&payload_names_ikev2, np));
+      return STF_FAIL + v2N_INVALID_SYNTAX;
+    }
+    seen |= s;
 	}
 
 	if (!in_struct(&pd->payload, sd, in_pbs, &pd->pbs))
 	{
-	    loglog(RC_LOG_SERIOUS, "malformed payload in packet");
-	    SEND_NOTIFICATION(PAYLOAD_MALFORMED);
-	    return STF_FAIL;
+      loglog(RC_LOG_SERIOUS, "malformed payload in packet");
+      return STF_FAIL + v2N_INVALID_SYNTAX;
 	}
 
 
 	DBG(DBG_PARSING
 	    , DBG_log("processing payload: %s (len=%u)\n"
-		      , enum_show(&payload_names, thisp)
+          , enum_show(&payload_names, np)
 		      , pd->payload.generic.isag_length));
 
 	/* place this payload at the end of the chain for this type */
 	{
 	    struct payload_digest **p;
 
-	    for (p = &md->chain[thisp]; *p != NULL; p = &(*p)->next)
+      for (p = &md->chain[np]; *p != NULL; p = &(*p)->next)
 		;
 	    *p = pd;
 	    pd->next = NULL;
 	}
 
-	np = pd->payload.generic.isag_np;
-
-	/* do payload-type specific things that need to be here. */
-	switch(thisp) {
+  switch(np) {
 	case ISAKMP_NEXT_v2E:
 	    np = ISAKMP_NEXT_NONE;
 	    break;
-	default:   /* nothing special */
+  default:
+      np = pd->payload.generic.isag_np;
 	    break;
 	}
 
 	pd++;
     }
+
+    if (req_payloads & ~seen) {
+   /* improperly repeated payload */
+   loglog(RC_LOG_SERIOUS,
+     "missing payload(s) (%s). Message dropped.",
+     bitnamesof(payload_name_ikev2_main, req_payloads & ~seen));
+   return STF_FAIL + v2N_INVALID_SYNTAX;
+     }
+
     md->digest_roof = pd;
     return STF_OK;
 }
 
+/* this stub is needed because struct state_v2_microcode is local to this file */
+stf_status ikev2_process_encrypted_payloads(struct msg_digest *md,
+            pb_stream   *in_pbs,
+            unsigned int np)
+{
+  return ikev2_process_payloads(md, in_pbs, np, md->svm->req_enc_payloads, md->svm->opt_enc_payloads);
+}
+
 /*
  * process an input packet, possibly generating a reply.
  *
@@ -346,7 +407,7 @@ process_v2_packet(struct msg_digest **mdp)
 
 	md->role = RESPONDER;
 
-	DBG(DBG_CONTROLMORE, DBG_log("I am IKE SA Responder"));
+	DBG(DBG_CONTROL, DBG_log("I am IKE SA Responder"));
 
 	st = find_state_ikev2_parent(md->hdr.isa_icookie
 				     , md->hdr.isa_rcookie);
@@ -357,36 +418,36 @@ process_v2_packet(struct msg_digest **mdp)
 	}
 
 	if(st) {
-	    /* if it is request from Initiator*/
-	    if(!(md->hdr.isa_flags & ISAKMP_FLAGS_R)) {
-		if(st->st_msgid_last_remotereq != INVALID_MSGID && st->st_msgid_last_remotereq > md->msgid_received){
+	    if(st->st_msgid_lastrecv >  md->msgid_received){
 		/* this is an OLD retransmit. we can't do anything */
-		openswan_log("received too old retransmit, ignoring: %u < %u"
-			     , md->msgid_received, st->st_msgid_last_remotereq);
+		openswan_log("received too old retransmit: %u < %u"
+			     , md->msgid_received, st->st_msgid_lastrecv);
 		return;
-		}
-		if(st->st_msgid_last_remotereq != INVALID_MSGID && st->st_msgid_last_remotereq == md->msgid_received){
+	    }
+	    if(st->st_msgid_lastrecv == md->msgid_received){
 		/* this is a recent retransmit. */
 		send_packet(st, "ikev2-responder-retransmit", FALSE);
 		return;
-		}
-	    }
-	    else
-	    {
-	    /* if it is response from Initiator*/
-		/* it seems that it should not happen
-		 * why a response will be retransmitted?
-		 */
-		if(st->st_msgid_last_localreq_ack!=INVALID_MSGID &&  st->st_msgid_last_localreq_ack >= md->msgid_received){
-		openswan_log("received an old response, ignoring: %u < %u"
-                             , md->msgid_received, st->st_msgid_last_localreq_ack);
-		}
+    /* PATRICK: I may have to uncomment the following block: */
+//		}
+//	    }
+//	    else
+//	    {
+//	    /* if it is response from Initiator*/
+//		/* it seems that it should not happen
+//		 * why a response will be retransmitted?
+//		 */
+//		if(st->st_msgid_last_localreq_ack!=INVALID_MSGID &&  st->st_msgid_last_localreq_ack >= md->msgid_received){
+//		openswan_log("received an old response, ignoring: %u < %u"
+//                             , md->msgid_received, st->st_msgid_last_localreq_ack);
+//		}
 	    }
 	    /* update lastrecv later on */
 	}
+	
     } else {
-        /* then I am the initiator, and this may be a reply or request */
-
+        /* then I am the initiator, and this is a reply */
+	
 	md->role = INITIATOR;
 
 	DBG(DBG_CONTROL, DBG_log("I am IKE SA Initiator"));
@@ -422,31 +483,39 @@ process_v2_packet(struct msg_digest **mdp)
 	}
 
 	if(st) {
-	    /* if it is request from Responder*/
-	    if(!(md->hdr.isa_flags & ISAKMP_FLAGS_R)) {
-		if(st->st_msgid_last_remotereq != INVALID_MSGID && st->st_msgid_last_remotereq > md->msgid_received){
-		/* this is an OLD retransmit. we can't do anything */
-		openswan_log("received too old retransmit, ignoring: %u < %u"
-			     , md->msgid_received, st->st_msgid_last_remotereq);
+	    /*
+	     * then there is something wrong with the msgid, so
+	     * maybe they retransmitted for some reason. 
+	     * Check if it's an old packet being returned, and
+	     * if so, drop it.
+	     * NOTE: in_struct() changed the byte order.
+	     */
+	    if(st->st_msgid_lastack != INVALID_MSGID
+	       && md->msgid_received <= st->st_msgid_lastack) {
+		/* it's fine, it's just a retransmit */
+		DBG(DBG_CONTROL, DBG_log("responding peer retransmitted msgid %u"
+					 , md->msgid_received));
 		return;
-		}
-		if(st->st_msgid_last_remotereq != INVALID_MSGID && st->st_msgid_last_remotereq == md->msgid_received){
-		/* this is a recent retransmit. */
-		send_packet(st, "ikev2-responder-retransmit", FALSE);
-		return;
-		}
-	    }
-	    else
-	    {
-	    /* if it is response from Responder*/
-		/* it seems that it should not happen
-		 * why a response will be retransmitted?
-		 */
-		if(st->st_msgid_last_localreq_ack!=INVALID_MSGID &&  st->st_msgid_last_localreq_ack >= md->msgid_received){
-		openswan_log("received an old response, ignoring: %u < %u"
-                             , md->msgid_received, st->st_msgid_last_localreq_ack);
-		}
+    /* PATRICK: I may have to uncomment the following block: */
+//		}
+//	    }
+//	    else
+//	    {
+//	    /* if it is response from Responder*/
+//		/* it seems that it should not happen
+//		 * why a response will be retransmitted?
+//		 */
+//		if(st->st_msgid_last_localreq_ack!=INVALID_MSGID &&  st->st_msgid_last_localreq_ack >= md->msgid_received){
+//		openswan_log("received an old response, ignoring: %u < %u"
+//                             , md->msgid_received, st->st_msgid_last_localreq_ack);
+//		}
 	    }
+#if 0
+	    openswan_log("last msgid ack is %u, received: %u"
+			 , st->st_msgid_lastack
+			 , md->msgid_received);
+	    return;
+#endif
 	}
     }
 
@@ -454,9 +523,10 @@ process_v2_packet(struct msg_digest **mdp)
     if(st) {
 
 	from_state = st->st_state;
+	DBG(DBG_CONTROL, DBG_log("state found and its state is (%s)", enum_show(&state_names, from_state)));
     }
 
-    for(svm = state_microcode_table; svm->state != STATE_IKEv2_ROOF; svm++) {
+    for(svm = v2_state_microcode_table; svm->state != STATE_IKEv2_ROOF; svm++) {
 	if(svm->flags & SMF2_STATENEEDED) {
 	    if(st==NULL) continue;
 	}
@@ -465,11 +535,20 @@ process_v2_packet(struct msg_digest **mdp)
 	}
 	if(svm->state != from_state) continue;
 	if(svm->recv_type != ix) continue;
+
+	/* I1 receiving NO_PROPOSAL ened up picking the wrong STATE_UNDEFINED state
+ 	   Since the wrong state is a responder, we just add a check for initiator,
+	   so we hit STATE_IKEv2_ROOF
+	 */
+	//if ( ((svm->flags&SMF2_INITIATOR) != 0) != ((md->hdr.isa_flags & ISAKMP_FLAGS_R) != 0) )
+        //        continue;
+	
 	/* must be the right state */
 	break;
     }
 
     if(svm->state == STATE_IKEv2_ROOF) {
+	DBG(DBG_CONTROL, DBG_log("ended up with STATE_IKEv2_ROOF"));
 
 	/* no useful state */
 	if(md->hdr.isa_flags & ISAKMP_FLAGS_I) {
@@ -481,42 +560,30 @@ process_v2_packet(struct msg_digest **mdp)
 	return;
     }
 
+    md->svm = svm;
+    md->from_state = from_state;
+    md->st = st;
+
     {
 	stf_status stf;
-	stf = ikev2_process_payloads(md, &md->message_pbs
-				     , from_state, md->hdr.isa_np);
+  stf = ikev2_process_payloads(md, &md->message_pbs,
+      md->hdr.isa_np,
+      svm->req_clear_payloads, svm->opt_clear_payloads);
+	
 	if(stf != STF_OK) {
 	    complete_v2_state_transition(mdp, stf);
 	    return;
 	}
     }
 
+    DBG(DBG_CONTROL, DBG_log("Now lets proceed with state specific processing"));
     DBG(DBG_PARSING,
 	if (pbs_left(&md->message_pbs) != 0)
 	    DBG_log("removing %d bytes of padding", (int) pbs_left(&md->message_pbs)));
 
     md->message_pbs.roof = md->message_pbs.cur;
 
-#if 0
-    /* check that all mandatory payloads appeared */
-    if (needed != 0)
-    {
-	loglog(RC_LOG_SERIOUS, "message for %s is missing payloads %s"
-	       , enum_show(&state_names, from_state)
-	       , bitnamesof(payload_name, needed));
-	SEND_NOTIFICATION(PAYLOAD_MALFORMED);
-	return;
-#endif
-
-    md->svm = svm;
-    md->from_state = from_state;
-    md->st  = st;
-
-    {
-	stf_status stf;
-	stf = (svm->processor)(md);
-	complete_v2_state_transition(mdp, stf);
-    }
+    complete_v2_state_transition(mdp, (svm->processor)(md));
 }
 
 bool
@@ -669,7 +736,7 @@ send_v2_notification_from_md(struct msg_digest *md UNUSED, u_int16_t type
 			 md->hdr.isa_icookie, md->hdr.isa_rcookie, data);
 }
 
-void ikev2_update_counters(struct msg_digest *md, enum ikev2_msgtype msgtype)
+void ikev2_update_counters(struct msg_digest *md)
 {
     struct state *pst= md->pst;
     struct state *st = md->st;
@@ -680,22 +747,36 @@ void ikev2_update_counters(struct msg_digest *md, enum ikev2_msgtype msgtype)
 	}
 	if(pst == NULL) pst = st;
     }
-
-
-	switch(msgtype) {
-	case req_sent:
-		pst->st_msgid_last_localreq = pst->st_msgid_last_localreq == INVALID_MSGID? 0 : pst->st_msgid_last_localreq + 1;
-		break;
-	case req_recd:
-		break;
-	case response_sent:
-		pst->st_msgid_last_remotereq = md->msgid_received;
-		break;
-	case response_recd:
-		pst->st_msgid_last_localreq_ack = md->msgid_received;
-		break;
-	default: break;
-	}
+    
+    /* PATRICK: I may have to switch the following blocks: */
+   /* Block 1 */
+    switch(md->role) {
+    case INITIATOR:
+	/* update lastuse values */
+	pst->st_msgid_lastack = md->msgid_received;
+	pst->st_msgid_nextuse = pst->st_msgid_lastack+1;
+	break;
+	
+    case RESPONDER:
+	pst->st_msgid_lastrecv= md->msgid_received;
+	break;
+    }
+  /* Block 2 */
+//	switch(msgtype) {
+//	case req_sent:
+//		pst->st_msgid_last_localreq = pst->st_msgid_last_localreq == INVALID_MSGID? 0 : pst->st_msgid_last_localreq + 1;
+//		break;
+//	case req_recd:
+//		break;
+//	case response_sent:
+//		pst->st_msgid_last_remotereq = md->msgid_received;
+//		break;
+//	case response_recd:
+//		pst->st_msgid_last_localreq_ack = md->msgid_received;
+//		break;
+//	default: break;
+//	}
+  /* End of block */
 }
 
 static void success_v2_state_transition(struct msg_digest **mdp)
@@ -713,6 +794,8 @@ static void success_v2_state_transition(struct msg_digest **mdp)
     change_state(st, svm->next_state);
     w = RC_NEW_STATE + st->st_state;
 
+    ikev2_update_counters(md);
+
 
     /* tell whack and log of progress */
     {
@@ -745,7 +828,12 @@ static void success_v2_state_transition(struct msg_digest **mdp)
 	    fmt_isakmp_sa_established(st, sadetails,sizeof(sadetails));
 	}
 
-	if (IS_CHILD_SA_ESTABLISHED(st) || IS_PARENT_SA_ESTABLISHED(st->st_state))
+  /* PATRICK: I may have to switch the following code blocks: */
+  /* Block 1 */
+	if (IS_CHILD_SA_ESTABLISHED(st))
+  /* Block 2 */
+	//if (IS_CHILD_SA_ESTABLISHED(st) || IS_PARENT_SA_ESTABLISHED(st->st_state))
+  /* End of blocks */
 	{
 	    /* log our success */
 	    w = RC_SUCCESS;
@@ -905,20 +993,11 @@ void complete_v2_state_transition(struct msg_digest **mdp
     struct msg_digest *md = *mdp;
     /* const struct state_v2_microcode *svm=md->svm; */
     struct state *st;
-    enum state_kind from_state;
+    enum state_kind from_state = STATE_UNDEFINED;
     const char *from_state_name;
 
-    /* advance the state */
-    DBG(DBG_CONTROL
-        , DBG_log("complete v2 state transition with %s"
-                  , enum_name(&stfstatus_name, result)));
-
-    /* this occur when IKE SA state is deleted already */
-    if(md->st == NULL) {
-	goto end;
-    }
-
     cur_state = st = md->st;	/* might have changed */
+
     /* passert(st);  // apparently on STF_TOOMUCH_CRYPTO we have no state? Needs fixing */
     /*
      * XXX/SML:  There is no need to abort here in all cases if state is
@@ -945,18 +1024,15 @@ void complete_v2_state_transition(struct msg_digest **mdp
     TCLCALLOUT("v2AdjustFailure", st, (st ? st->st_connection : NULL), md);
     result = md->result;
 
+    /* advance the state */
+    DBG(DBG_CONTROL
+	, DBG_log("complete v2 state transition with %s"
+		  , enum_name(&stfstatus_name, (result > STF_FAIL) ? STF_FAIL : result)));
 
     switch(result) {
     case STF_IGNORE:
 	break;
 
-    case STF_INLINE:         /* this is second time through complete
-			      * state transition, so the MD has already
-			      * been freed.
-			      0				  */
-	*mdp = NULL;
-	break;
-
     case STF_SUSPEND:
 	/* update the previous packet history */
 	/* IKEv2 XXX */ /* update_retransmit_history(st, md); */
@@ -965,6 +1041,13 @@ void complete_v2_state_transition(struct msg_digest **mdp
 	*mdp = NULL;
 	break;
 
+    case STF_INLINE:         /* mcr: this is second time through complete
+			      * state transition, so the MD has already
+			      * been freed.
+			      0				  */
+			      *mdp = NULL;
+			      /* fall through to STF_OK */
+
     case STF_OK:
 	/* advance the state */
 	passert(st);
@@ -1020,31 +1103,43 @@ void complete_v2_state_transition(struct msg_digest **mdp
 		  , from_state_name
 		  , enum_name(&ipsec_notification_names, md->note));
 
-#if 0
 	if(md->note > 0) {
 		/* only send a notify is this packet was a question, not if it was an answer */
 		if(!(md->hdr.isa_flags & ISAKMP_FLAGS_R)) {
 		     SEND_NOTIFICATION(md->note);
 		}
 	}
-#endif
 
 	DBG(DBG_CONTROL,
 	    DBG_log("state transition function for %s failed: %s"
 		    , from_state_name
 		    , (md->note) ? enum_name(&ipsec_notification_names, md->note) : "<no reason given>" ));
     }
-end:;
 }
 
 v2_notification_t
 accept_v2_nonce(struct msg_digest *md, chunk_t *dest, const char *name)
 {
-    return accept_nonce(md, dest, name, ISAKMP_NEXT_v2Ni);
+	pb_stream *nonce_pbs;
+	size_t len;
+
+	if(md->chain[ISAKMP_NEXT_v2Ni] == NULL) {
+		loglog(RC_LOG_SERIOUS, "missing nonce Ni");
+		return v2N_INVALID_SYNTAX;
+	}
+
+	nonce_pbs = &md->chain[ISAKMP_NEXT_v2Ni]->pbs;
+	len = pbs_left(nonce_pbs);
+
+	if (len < MINIMUM_NONCE_SIZE || MAXIMUM_NONCE_SIZE < len) {
+		loglog(RC_LOG_SERIOUS, "%s length not between %d and %d",
+			name, MINIMUM_NONCE_SIZE, MAXIMUM_NONCE_SIZE);
+		return v2N_INVALID_SYNTAX; /* ??? */
+	}
+	clonereplacechunk(*dest, nonce_pbs->cur, len, "nonce");
+	return NOTHING_WRONG;
 }
 
-
-
 /*
  * Local Variables:
  * c-basic-offset:4
