@@ -1362,6 +1362,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
     IPsecSAref_t refhim = st->st_refhim;
     IPsecSAref_t new_refhim = IPSEC_SAREF_NULL;
 
+    const char *inbound_str = inbound ? "inbound" : "outbound";
+
     /* SPIs, saved for spigrouping or undoing, if necessary */
     struct kernel_sa
         said[EM_MAXRELSPIS],
@@ -1658,7 +1660,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 #endif
 
 	DBG(DBG_CRYPT
-	    , DBG_log("looking for alg with transid: %d keylen: %d auth: %d\n"
+	    , DBG_log("looking for %s alg with transid: %d keylen: %d auth: %d\n"
+                      , inbound_str
 		      , st->st_esp.attrs.transattrs.encrypt
 		      , st->st_esp.attrs.transattrs.enckeylen
 		      , st->st_esp.attrs.transattrs.integ_hash));
@@ -1791,8 +1794,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 	said_next->sa_lifetime = c->sa_ipsec_life_seconds;
 
 	DBG(DBG_CRYPT,
-	  DBG_dump("esp enckey:",  said_next->enckey,  said_next->enckeylen);
-	  DBG_dump("esp authkey:", said_next->authkey, said_next->authkeylen);
+            DBG_dump("esp %s enckey:",  said_next->enckey,  said_next->enckeylen);
+            DBG_dump("esp %s authkey:", said_next->authkey, said_next->authkeylen);
 	);
 
 	if(inbound) {
@@ -2033,31 +2036,6 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 			      , st->st_connection->policy_label
 #endif
 			      );
-
-	if(st->st_ikev2) {
-	   struct spd_route *sr = &c->spd;
-	   for(sr = sr->next; sr != NULL; sr = sr->next) {
-
-            /* MCR - should be passed a spd_eroute structure here */
-            (void) raw_eroute(&sr->that.host_addr   /* this_host */
-			      , &sr->that.client    /* this_client */
-                              , &sr->this.host_addr /* that_host */
-			      , &sr->this.client    /* that_client */
-                              , inner_spi              /* spi */
-			      , proto                  /* proto */
-                              , sr->this.protocol   /* transport_proto */
-                              , esatype                /* esatype */
-                              , proto_info             /* " */
-			      , 0                      /* lifetime */
-                              , ERO_ADD_INBOUND        /* op */
-			      , "add inbound"        /* opname */
-#ifdef HAVE_LABELED_IPSEC
-			      , st->st_connection->policy_label
-#endif
-			      );
-	   }
-	}
-
         }
     }
 
@@ -2133,7 +2111,6 @@ teardown_half_ipsec_sa(struct state *st, bool inbound)
      * first one found.  It may or may not be the only one.
      */
     struct connection *c = st->st_connection;
-    struct spd_route *sr;
     struct {
         unsigned proto;
         struct ipsec_proto_info *info;
@@ -2159,30 +2136,28 @@ teardown_half_ipsec_sa(struct state *st, bool inbound)
 			  );
     }
 
-    if(st->st_ikev2) {
-        for(sr = &c->spd; sr; sr=sr->next) {
-            if (kernel_ops->inbound_eroute && inbound
-                && sr->eroute_owner == SOS_NOBODY)
-                {
-                    (void) raw_eroute(&sr->that.host_addr, &sr->that.client
-                                      , &sr->this.host_addr, &sr->this.client
-                                      , 256
-                                      , IPSEC_PROTO_ANY
-                                      , sr->this.protocol
-                                      , ET_UNSPEC
-                                      , null_proto_info, 0
-                                      , ERO_DEL_INBOUND, "delete inbound"
-#ifdef HAVE_LABELED_IPSEC
-                                      , c->policy_label
-#endif
-                                      );
-                }
-        }
-    }
-
-
-
-
+    /* PATRICK: I may have to uncomment the following block: */
+//    if(st->st_ikev2) {
+//        for(sr = &c->spd; sr; sr=sr->next) {
+//            if (kernel_ops->inbound_eroute && inbound
+//                && sr->eroute_owner == SOS_NOBODY)
+//                {
+//                    (void) raw_eroute(&sr->that.host_addr, &sr->that.client
+//                                      , &sr->this.host_addr, &sr->this.client
+//                                      , 256
+//                                      , IPSEC_PROTO_ANY
+//                                      , sr->this.protocol
+//                                      , ET_UNSPEC
+//                                      , null_proto_info, 0
+//                                      , ERO_DEL_INBOUND, "delete inbound"
+//#ifdef HAVE_LABELED_IPSEC
+//                                      , c->policy_label
+//#endif
+//                                      );
+//                }
+//        }
+//    }
+//
     if (!kernel_ops->grp_sa)
     {
         if (st->st_ah.present)
@@ -2286,12 +2261,12 @@ init_kernel(void)
     case USE_NETKEY:
 	if (stat("/proc/net/pfkey", &buf) == 0) {
 	    kern_interface = USE_NETKEY;
-	    openswan_log("Using Linux 2.6 IPsec interface code on %s (experimental code)"
+	    openswan_log("Using Linux XFRM/NETKEY IPsec interface code on %s"
 			 , kversion);
 	    kernel_ops = &netkey_kernel_ops;
 	    break;
 	} else
-	    openswan_log("No Kernel NETKEY interface detected");
+	    openswan_log("No Kernel XFRM/NETKEY interface detected");
 	/* FALL THROUGH */
 #endif
 
@@ -2881,7 +2856,7 @@ install_ipsec_sa(struct state *st, bool inbound_also USED_BY_KLIPS)
 	if(!setup_half_ipsec_sa(st, FALSE)) {
 	    return FALSE;
 	}
-	DBG(DBG_KLIPS, DBG_log("set up outoing SA, ref=%u/%u", st->st_ref, st->st_refhim));
+	DBG(DBG_KLIPS, DBG_log("set up outgoing SA, ref=%u/%u", st->st_ref, st->st_refhim));
 	st->st_outbound_done = TRUE;
     }
 
