@@ -544,6 +544,13 @@ netlink_raw_eroute(const ip_address *this_host
     DBG(DBG_NETKEY,
 	DBG_log("satype(%d) is not used in netlink_raw_eroute.",satype));
 
+
+    /* log warning for RFC-breaking implementation in NETKEY/XFRM stack */
+    if( (proto_info[0].encapsulation != ENCAPSULATION_MODE_TUNNEL)
+	&& (transport_proto != 0)) {
+	   DBG_log("warning: NETKEY/XFRM in transport mode accepts ALL encrypted protoport packets between the hosts in violation of RFC 4301, Section 5.2");
+    }
+
     /* Bug #1004 fix.
      * There really isn't "client" with NETKEY and transport mode
      * so eroute must be done to natted, visible ip. If we don't hide
@@ -823,11 +830,11 @@ netlink_add_sa(struct kernel_sa *sa, bool replace)
 	req.p.sel.dport = portof(&sa->dst_client->addr);
 
 	/* As per RFC 4301/5996, icmp type is put in the most significant 8 bits
-	* and icmp code is in the least significant 8 bits of port field. 
-	* Although Openswan does not have any configuration options for 
-	* icmp type/code values, it is possible to specify icmp type and code 
-	* using protoport option. For example, icmp echo request (type 8/code 0) 
-	* needs to be encoded as 0x0800 in the port field and can be specified 
+	* and icmp code is in the least significant 8 bits of port field.
+	* Although Openswan does not have any configuration options for
+	* icmp type/code values, it is possible to specify icmp type and code
+	* using protoport option. For example, icmp echo request (type 8/code 0)
+	* needs to be encoded as 0x0800 in the port field and can be specified
 	* as left/rightprotoport=icmp/2048. Now with NETKEY, icmp type and code
 	* need to be passed as source and destination ports, respectively.
 	* therefore, this code extracts upper 8 bits and lower 8 bits and puts
@@ -1156,6 +1163,8 @@ linux_pfkey_add_aead(void)
 {
 	struct sadb_alg alg;
 
+	alg.sadb_alg_reserved = 0;
+
 	alg.sadb_alg_ivlen = 8;
 	alg.sadb_alg_minbits = 128 + 4 * BITS_PER_BYTE;
 	alg.sadb_alg_maxbits = 256 + 4 * BITS_PER_BYTE;
