@@ -740,7 +740,7 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     r = &reqbuf[0];
 
     if(r->pcr_len > sizeof(reqbuf)) {
-	loglog(RC_LOG_SERIOUS, "helper(%d) pid=%d screwed up length: %lu > %lu, killing it"
+	loglog(RC_LOG_SERIOUS, "helper(%d) pid=%lu screwed up length: %lu > %lu, killing it"
 	       , w->pcw_helpernum
 	       , w->pcw_pid, (unsigned long)r->pcr_len
                , (unsigned long)sizeof(reqbuf));
@@ -995,7 +995,7 @@ static void cleanup_crypto_helper(struct pluto_crypto_worker *w
 				  , int status)
 {
     if(w->pcw_pipe) {
-	loglog(RC_LOG_SERIOUS, "closing helper(%u) pid=%d fd=%d exit=%d"
+	loglog(RC_LOG_SERIOUS, "closing helper(%u) pid=%lu fd=%d exit=%d"
 	       , w->pcw_helpernum, w->pcw_pid, w->pcw_pipe, status);
 	close(w->pcw_pipe);
     }
