@@ -1,7 +1,7 @@
 /* Pluto main program
  * Copyright (C) 1997      Angelos D. Keromytis.
  * Copyright (C) 1998-2001 D. Hugh Redelmeier.
- * Copyright (C) 2003-2008 Michael C Richardson <mcr@xelerance.com>
+ * Copyright (C) 2001-2013 Michael C Richardson <mcr@xelerance.com>
  * Copyright (C) 2003-2010 Paul Wouters <paul@xelerance.com>
  * Copyright (C) 2007 Ken Bantoft <ken@xelerance.com>
  * Copyright (C) 2008-2009 David McCullough <david_mccullough@securecomputing.com>
@@ -140,6 +140,7 @@ usage(const char *mess)
 	    " \\\n\t"
 	    "[--nofork]"
 	    " [--stderrlog]"
+	    " [--plutostderrlogtime]"
 	    " [--force_busy]"
 	    " [--nocrsend]"
 	    " [--strictcrlpolicy]"
@@ -306,6 +307,7 @@ enum kernel_interface kern_interface = AUTO_PICK;
 char **global_argv;
 int    global_argc;
 bool   log_to_stderr_desired = FALSE;
+bool   log_with_timestamp_desired = FALSE;
 
 #ifdef HAVE_LABELED_IPSEC
 u_int16_t secctx_attr_value=SECCTX;
@@ -374,6 +376,7 @@ main(int argc, char **argv)
 	    { "optionsfrom", required_argument, NULL, '+' },
 	    { "nofork", no_argument, NULL, 'd' },
 	    { "stderrlog", no_argument, NULL, 'e' },
+	    { "plutostderrlogtime", no_argument, NULL, 't' },
 	    { "noklips", no_argument, NULL, 'n' },
 	    { "use-nostack",  no_argument, NULL, 'n' },
 	    { "use-none",     no_argument, NULL, 'n' },
@@ -489,12 +492,8 @@ main(int argc, char **argv)
 
 	case 'v':	/* --version */
 	    {
-		const char **sp = ipsec_copyright_notice();
-
 		printf("%s%s\n", ipsec_version_string(),
 				 compile_time_interop_options);
-		for (; *sp != NULL; sp++)
-		    puts(*sp);
 	    }
 	    exit(0);	/* not exit_pluto because we are not initialized yet */
 	    break;	/* not actually reached */
@@ -544,6 +543,10 @@ main(int argc, char **argv)
 	    log_to_stderr_desired = TRUE;
 	    continue;
 
+	case 't':	/* --plutostderrlogtime */
+	    log_with_timestamp_desired = TRUE;
+	    continue;
+
 	case 'G':       /* --use-auto */
 	    kern_interface = AUTO_PICK;
 	    continue;
@@ -749,9 +752,12 @@ main(int argc, char **argv)
 
     /* select between logging methods */
 
-    if (log_to_stderr_desired)
+    if (log_to_stderr_desired) {
 	log_to_syslog = FALSE;
-    else
+	if (log_with_timestamp_desired)
+	   log_with_timestamp = TRUE;
+    }
+    else 
 	log_to_stderr = FALSE;
 
 #ifdef DEBUG
