@@ -32,7 +32,6 @@
 #include "x509.h"
 #include "pgp.h"
 #include "certs.h"
-#include "smartcard.h"
 #ifdef XAUTH_USEPAM
 #include <security/pam_appl.h>
 #endif
@@ -59,7 +58,7 @@ void
 print_sa_attr_oakley(struct db_attr *at)
 {
     const struct enum_names *en = NULL;
-	
+
     if(at->type.oakley == 0) {
 	return;
     }
@@ -76,7 +75,7 @@ void
 print_sa_attr_ipsec(struct db_attr *at)
 {
     const struct enum_names *en = NULL;
-	
+
     if(at->type.ipsec == 0) {
 	return;
     }
@@ -142,16 +141,16 @@ sa_print(struct db_sa *f)
 static void
 print_sa_v2_attr(struct db_attr *at)
 {
-    const struct enum_names *en = NULL;
-	
+    //const struct enum_names *en = NULL;
+
     if(at->type.ikev2 == 0) {
 	return;
     }
 
-    en = NULL; /* XXX */
+    //en = NULL; /* XXX */
     printf("        type: %u(%s) val: %u(%s)\n"
 	   , at->type.ikev2, "" /*enum_name(&oakley_attr_names, at->type+ISAKMP_ATTR_AF_TV)*/
-	   , at->val,  en ? enum_name(en, at->val) : "unknown");
+	   , at->val, "unknown");
 }
 
 void
@@ -210,7 +209,7 @@ sa_v2_print(struct db_sa *f)
 	}
 }
 
-    
+
 /*
  * Local Variables:
  * c-style: pluto
