@@ -165,7 +165,6 @@ struct traffic_selector {
     u_int16_t endport;
     ip_address low;
     ip_address high;
-    struct traffic_selector *next;
 };
 
 #ifdef HAVE_LABELED_IPSEC
@@ -251,13 +250,12 @@ struct state
     struct msgid_list  *st_used_msgids;        /* used-up msgids */
 
     /* IKEv2 things */
-    /* message ID sequence for things we send (as request) */
-    msgid_t            st_msgid_last_localreq;       /* last one sent as request by us */
-    msgid_t            st_msgid_last_localreq_ack;       /* last one sent acked by peer */
+    /* message ID sequence for things we send (as initiator) */
+    msgid_t            st_msgid_lastack;       /* last one peer acknowledged */
+    msgid_t            st_msgid_nextuse;       /* next one to use */
 
-
-    /* message ID sequence for things we receive (as response) */
-   msgid_t            st_msgid_last_remotereq;       /* last remote request sent by peer */
+    /* message ID sequence for things we receive (as responder) */
+    msgid_t            st_msgid_lastrecv;      /* last one peer sent */
 
 
     /* symmetric stuff */
